unit uOptions;
{$INCLUDE 'Plugin.inc'}

interface

uses
  {$INCLUDE 'DelphiFarUnits.inc'} ;

type
  TOptions = class(TFarConfig)
  public
    type TCursorMovementOption = (cmoMinimizeScrolling, cmoPrioritizeSourceFileName, cmoPrioritizeDestinationFileName, cmoDestinationFileNameOnly);
    type TSmartEndBehaviorOption = (sebExtStartLineEnd, sebExtStartExtEnd, sebExtEndLineEnd);
  private
    fTinyHelp: boolean;
    fSelectRenamedFiles: boolean;
    fDisableInternalHotkeys: boolean;
    fSelectDestinationFilename: boolean;
    fCursorMovement: TCursorMovementOption;
    fSmartEndBehavior: TSmartEndBehaviorOption;
    fUpdateDescriptions: boolean;
    fDescriptionFiles: string;
    procedure SetDisableInternalHotkeys(const Value: boolean);
    procedure SetSelectRenamedFiles(const Value: boolean);
    procedure SetTinyHelp(const Value: boolean);
    procedure SetSelectDestinationFilename(const Value: boolean);
    procedure SetCursorMovement(const Value: TCursorMovementOption);
    procedure SetUpdateDescriptions(const Value: boolean);
    procedure SetDescriptionFiles(const Value: string);
    procedure SetSmartEndBehavior(const Value: TSmartEndBehaviorOption);
  protected
    procedure SetConfigOption(var OptionVar: TCursorMovementOption; const NewValue: TCursorMovementOption); overload;
    procedure SetConfigOption(var OptionVar: TSmartEndBehaviorOption; const NewValue: TSmartEndBehaviorOption); overload;
    function CursorMovementOptionToInt(const Value: TCursorMovementOption): integer;
    function IntToCursorMovementOption(const Value: integer; const Default: TCursorMovementOption): TCursorMovementOption;
    function SmartEndBehaviorOptionToInt(const Value: TSmartEndBehaviorOption): integer;
    function IntToSmartEndBehaviorOption(const Value: integer; const Default: TSmartEndBehaviorOption): TSmartEndBehaviorOption;
  public
    constructor Create;
    destructor Destroy; override;
    procedure Init;
    procedure Load;
    procedure Save(Force: boolean = False);
    procedure Assign(Options: TOptions);
    property TinyHelp: boolean read fTinyHelp write SetTinyHelp;
    property SelectRenamedFiles: boolean read fSelectRenamedFiles write SetSelectRenamedFiles;
    property DisableInternalHotkeys: boolean read fDisableInternalHotkeys write SetDisableInternalHotkeys;
    property SelectDestinationFilename: boolean read fSelectDestinationFilename write SetSelectDestinationFilename;
    property CursorMovement: TCursorMovementOption read fCursorMovement write SetCursorMovement;
    property SmartEndBehavior: TSmartEndBehaviorOption read fSmartEndBehavior write SetSmartEndBehavior;
    property UpdateDescriptions: boolean read fUpdateDescriptions write SetUpdateDescriptions;
    property DescriptionFiles: string read fDescriptionFiles write SetDescriptionFiles;
  end;

implementation

uses
  uMessages;

{ TOptions }

procedure TOptions.Assign(Options: TOptions);
begin
  TinyHelp := Options.TinyHelp;
  SelectRenamedFiles := Options.SelectRenamedFiles;
  DisableInternalHotkeys := Options.DisableInternalHotkeys;
  SelectDestinationFilename := Options.SelectDestinationFilename;
  CursorMovement := Options.CursorMovement;
  SmartEndBehavior := Options.SmartEndBehavior;
  UpdateDescriptions := Options.UpdateDescriptions;
  DescriptionFiles := Options.DescriptionFiles;
end;

constructor TOptions.Create;
begin
  inherited Create('FRename3', False);
  Init;
end;

function TOptions.CursorMovementOptionToInt(const Value: TCursorMovementOption): integer;
begin
  case Value of
    cmoMinimizeScrolling:
      Result := 1;
    cmoPrioritizeSourceFileName:
      Result := 2;
    cmoPrioritizeDestinationFileName:
      Result := 3;
    cmoDestinationFileNameOnly:
      Result := 4;
    else
      Result := 0;
  end;
end;

destructor TOptions.Destroy;
begin
  inherited;
end;

procedure TOptions.Init;
begin
  TinyHelp := False;
  SelectRenamedFiles := False;
  DisableInternalHotkeys := False;
  SelectDestinationFilename := False;
  CursorMovement := TCursorMovementOption.cmoPrioritizeDestinationFileName;
  SmartEndBehavior := TSmartEndBehaviorOption.sebExtStartLineEnd;
  UpdateDescriptions := False;
  DescriptionFiles := 'descript.ion,files.bbs';
  SetChanged;
end;

function TOptions.IntToCursorMovementOption(const Value: integer; const Default: TCursorMovementOption): TCursorMovementOption;
begin
  case Value of
    1:   Result := cmoMinimizeScrolling;
    2:   Result := cmoPrioritizeSourceFileName;
    3:   Result := cmoPrioritizeDestinationFileName;
    4:   Result := cmoDestinationFileNameOnly;
    else Result := Default;
  end;
end;

function TOptions.IntToSmartEndBehaviorOption(const Value: integer; const Default: TSmartEndBehaviorOption): TSmartEndBehaviorOption;
begin
  case Value of
    1:   Result := sebExtStartLineEnd;
    2:   Result := sebExtStartExtEnd;
    3:   Result := sebExtEndLineEnd;
    else Result := Default;
  end;
end;

procedure TOptions.Load;
begin
  if Open then
    try
      TinyHelp := ReadBoolean('TinyHelp', TinyHelp);
      SelectRenamedFiles := ReadBoolean('SelectRenamedFiles', SelectRenamedFiles);
      DisableInternalHotkeys := ReadBoolean('DisableInternalHotkeys', DisableInternalHotkeys);
      SelectDestinationFilename := ReadBoolean('SelectDestinationFilename', SelectDestinationFilename);
      CursorMovement := IntToCursorMovementOption(ReadInteger('CursorMovement', -1), CursorMovement);
      SmartEndBehavior := IntToSmartEndBehaviorOption(ReadInteger('SmartEndBehavior', -1), SmartEndBehavior);
      UpdateDescriptions := ReadBoolean('UpdateDescriptions', UpdateDescriptions);
      DescriptionFiles := ReadString('DescriptionFiles', DescriptionFiles);
      ClearChanged;
    finally
      Close;
    end;
end;

procedure TOptions.Save(Force: boolean);
begin
  if not (Force or Changed) then
    Exit;
  if Open then
    try
      WriteBoolean('TinyHelp', TinyHelp);
      WriteBoolean('SelectRenamedFiles', SelectRenamedFiles);
      WriteBoolean('DisableInternalHotkeys', DisableInternalHotkeys);
      WriteBoolean('SelectDestinationFilename', SelectDestinationFilename);
      WriteInteger('CursorMovement', CursorMovementOptionToInt(CursorMovement));
      WriteInteger('SmartEndBehavior', SmartEndBehaviorOptionToInt(SmartEndBehavior));
      WriteBoolean('UpdateDescriptions', UpdateDescriptions);
      WriteString('DescriptionFiles', DescriptionFiles);
      ClearChanged;
    finally
      Close;
    end;
end;

procedure TOptions.SetConfigOption(var OptionVar: TCursorMovementOption; const NewValue: TCursorMovementOption);
begin
  if NewValue <> OptionVar then
  begin
    OptionVar := NewValue;
    SetChanged;
  end;
end;

procedure TOptions.SetConfigOption(var OptionVar: TSmartEndBehaviorOption; const NewValue: TSmartEndBehaviorOption);
begin
  if NewValue <> OptionVar then
  begin
    OptionVar := NewValue;
    SetChanged;
  end;
end;

procedure TOptions.SetCursorMovement(const Value: TCursorMovementOption);
begin
  SetConfigOption(fCursorMovement, Value);
end;

procedure TOptions.SetDescriptionFiles(const Value: string);
begin
  SetConfigOption(fDescriptionFiles, Value);
end;

procedure TOptions.SetDisableInternalHotkeys(const Value: boolean);
begin
  SetConfigOption(fDisableInternalHotkeys, Value);
end;

procedure TOptions.SetSelectDestinationFilename(const Value: boolean);
begin
  SetConfigOption(fSelectDestinationFilename, Value);
end;

procedure TOptions.SetSelectRenamedFiles(const Value: boolean);
begin
  SetConfigOption(fSelectRenamedFiles, Value);
end;

procedure TOptions.SetSmartEndBehavior(const Value: TSmartEndBehaviorOption);
begin
  SetConfigOption(fSmartEndBehavior, Value);
end;

procedure TOptions.SetTinyHelp(const Value: boolean);
begin
  SetConfigOption(fTinyHelp, Value);
end;

procedure TOptions.SetUpdateDescriptions(const Value: boolean);
begin
  SetConfigOption(fUpdateDescriptions, Value);
end;

function TOptions.SmartEndBehaviorOptionToInt(const Value: TSmartEndBehaviorOption): integer;
begin
  case Value of
    sebExtStartLineEnd:
      Result := 1;
    sebExtStartExtEnd:
      Result := 2;
    sebExtEndLineEnd:
      Result := 3;
    else
      Result := 0;
  end;
end;

end.
